<?php
namespace tests\Zeedhi\Tools\DatabaseReader\Adapters;

use Zeedhi\Tools\DatabaseReader\DatabaseReader;
use Zeedhi\Tools\DatabaseReader\DTO\Column;
use Zeedhi\Tools\DatabaseReader\DTO\Constraints\ForeignKey;
use Doctrine\DBAL\DriverManager;

class OracleTest extends \PHPUnit_Framework_TestCase {

    /** @var DatabaseReader */
    protected $reader;

    protected function setUp() {
        if (!extension_loaded('oci8')) {
            $this->markTestSkipped('The ' . __CLASS__ . ' requires the use of oci8 extension.');
        }

        $connectionToRead = DriverManager::getConnection(array(
            'driver'    => 'oci8',
            'user'      => 'USR_ORG_20',
            'password'  => 'teknisa',
            'host'      => '192.168.122.5',
            'port'      => '1521',
            'dbname'    => 'pdborcl',
            'service'   => true
        ));
        $this->reader = DatabaseReader::factoryFromConnection($connectionToRead);
    }


    public function testGetAllTableNames() {
        $expectedTableNames = array(
            "COUNTRIES",
            "DEPARTMENTS",
            "EMPLOYEES",
            "JOB_HISTORY",
            "JOBS",
            "LOCATIONS",
            "REGIONS",
            "TEST_LONG_RAW",
        );

        $tableNames = $this->reader->getAllTableNames();
        foreach ($expectedTableNames as $expectedTableName) {
            $this->assertContains($expectedTableName, $tableNames, "Table expected to be found.");
        }
    }

    public function testGetTableThatDoesNotExist() {
        $tableName = "INVALID_NAME";
        $this->setExpectedException("\\Zeedhi\\Tools\\DatabaseReader\\Exception", "Table {$tableName} was not found in current database.");
        $this->reader->getTable($tableName);
    }

    public function testGetTable() {
        $DTOTable = $this->reader->getTable("COUNTRIES");
        $this->assertInstanceOf("\\Zeedhi\\Tools\\DatabaseReader\\DTO\\Table", $DTOTable, "Return must be a instance of DTO\\TAble.");
        $this->assertEquals("COUNTRIES", $DTOTable->getName(), "Table name must be 'COUNTRIES'.");
    }

    public function assertNullOrNumeric($value, $message = '') {
        $this->assertTrue(is_numeric($value) || is_null($value), $message);
    }

    public function assertDTOColumn(Column $DTOColumn) {
        $columnTypeList = array(
            Column::DATA_TYPE_BINARY,
            Column::DATA_TYPE_BOOLEAN,
            Column::DATA_TYPE_DATE,
            Column::DATA_TYPE_DATETIME,
            Column::DATA_TYPE_FLOAT,
            Column::DATA_TYPE_INTEGER,
            Column::DATA_TYPE_STRING,
            Column::DATA_TYPE_STRING_FIXED,
        );
        $this->assertNotEmpty($DTOColumn->getName(), "Column name can't be empty.");
        $this->assertNullOrNumeric($DTOColumn->getDataPrecision(), "DataPrecision must be null or numeric value.");
        $this->assertNullOrNumeric($DTOColumn->getDataLength(), "DataLength must be null or numeric value.");
        $this->assertNullOrNumeric($DTOColumn->getDataScale(), "DataScale must be null or numeric value.");
        $this->assertContains($DTOColumn->getNullAble(), array("Y", "N"), "Column nullable must be a boolean");
        $this->assertContains($DTOColumn->getDataType(), $columnTypeList, "Column must have a DataType in the list.");
    }

    public function testGetColumnsFromTable() {
        $DTOColumns = $dbReader = $this->reader->getColumnsFromTable("COUNTRIES");
        $this->assertContainsOnlyInstancesOf("\\Zeedhi\\Tools\\DatabaseReader\\DTO\\Column", $DTOColumns);
        foreach($DTOColumns as $DTOColumn) {
            $this->assertDTOColumn($DTOColumn);
        }
    }

    public function testGetColumnsFromInvalidTable() {
        $tableName = "INVALID_NAME";
        $this->setExpectedException("\\Zeedhi\\Tools\\DatabaseReader\\Exception", "Table {$tableName} was not found in current database.");
        $this->reader->getColumnsFromTable($tableName);
    }

    public function testGetPrimaryKeyFromTable() {
        $primaryKey = $this->reader->getPrimaryKeyFromTable("COUNTRIES");
        $this->assertInstanceOf("\\Zeedhi\\Tools\\DatabaseReader\\DTO\\Constraints\\PrimaryKey", $primaryKey);
    }

    public function testGetPrimaryKeyFromInvalidTable() {
        $tableName = "INVALID_NAME";
        $this->setExpectedException("\\Zeedhi\\Tools\\DatabaseReader\\Exception", "Table {$tableName} was not found in current database.");
        $this->reader->getPrimaryKeyFromTable($tableName);
    }

    /**
     * @param ForeignKey $foreignKey
     */
    public function assertForeignKey($foreignKey) {
        static $deleteRuleList = array(
            ForeignKey::DELETE_RULE_CASCADE,
            ForeignKey::DELETE_RULE_RESTRICT,
            ForeignKey::DELETE_RULE_SET_NULL,
            ForeignKey::DELETE_RULE_NO_ACTION,
        );

        $this->assertNotNull($foreignKey->getReferencedConstraintName());
        $this->assertContains($foreignKey->getDeleteRule(), $deleteRuleList, "ForeignKey must have a delete rule in list.");
    }

    public function testGetForeignKeyFromTable() {
        $foreignKeys = $this->reader->getForeignKeysFromTable("COUNTRIES");
        $this->assertContainsOnlyInstancesOf("\\Zeedhi\\Tools\\DatabaseReader\\DTO\\Constraints\\ForeignKey", $foreignKeys);
        foreach ($foreignKeys as $foreignKey) {
            $this->assertForeignKey($foreignKey);
        }
    }

    public function testGetForeignKeyFromInvalidTable() {
        $tableName = "INVALID_NAME";
        $this->setExpectedException("\\Zeedhi\\Tools\\DatabaseReader\\Exception", "Table {$tableName} was not found in current database.");
        $this->reader->getForeignKeysFromTable($tableName);
    }

    public function testGetUniqueConstraintFromTable() {
        $uniqueConstraints = $this->reader->getUniqueConstraintsFromTable("COUNTRIES");
        $this->assertContainsOnlyInstancesOf("\\Zeedhi\\Tools\\DatabaseReader\\DTO\\Constraints\\Unique", $uniqueConstraints);
        foreach ($uniqueConstraints as $uniqueConstraint) {
            $this->assertConsColumnCollection($uniqueConstraint->getConsColumns());
        }
    }

    public function testGetUniqueConstraintFromInvalidTable() {
        $tableName = "INVALID_NAME";
        $this->setExpectedException("\\Zeedhi\\Tools\\DatabaseReader\\Exception", "Table {$tableName} was not found in current database.");
        $this->reader->getUniqueConstraintsFromTable($tableName);
    }

    /**
     * @param \Zeedhi\Tools\DatabaseReader\DTO\Constraints\ConsColumn[] $DTOConsColumns
     */
    public function assertConsColumnCollection($DTOConsColumns) {
        $this->assertContainsOnlyInstancesOf("\\Zeedhi\\Tools\\DatabaseReader\\DTO\\Constraints\\ConsColumn", $DTOConsColumns);
        foreach ($DTOConsColumns as $DTOConsColumn) {
            $this->assertNotEmpty($DTOConsColumn->getColumnName());
            $this->assertTrue(is_numeric($DTOConsColumn->getPosition()));
        }
    }
}